/* *****************************************
 *  File name: I2C_EEPROM.c
 *  Notes: 48MHz内部クロック
 *        PWM出力 RC5  PushSW  RA5
 * Target PIC16F1459
 * Created on May 5, 2020, 6:31 PM
 ***************************************** */
//
#include <xc.h>
#include "I2C_EEPROM.h"
#define _XTAL_FREQ 32000000

#define SlaveAdd_R 0xA1
#define SlaveAdd_W 0xA0

//--------------- I2C Routines -------------------
void i2c_MasterInit(const unsigned long baud){
    // pin設定 ------------------------------------------------
    TRISB4 = 1;           // RB4を入力
    TRISB6 = 1;           // RB6を入力
    ANSB4  = 0;           // RB4をデジタル
    ANSB6  = 0;           // RB6をデジタル
    ODCB4  = 1;           // RB4をオープンドレイン
    ODCB6  = 1;           // RB6をオープンドレイン

    // PPS設定　-----------------------------------------------
    SSP1CLKPPS = 0x0E;    // RB6をCLK入力に指定
    RB6PPS     = 24;      // RB6をCLK出力に指定
    SSP1DATPPS = 0x0C;    // RB4をDATに入力指定
    RB4PPS     = 25;      // RB4をDAT出力に指定

    // SSP1設定 -----------------------------------------------
    SSP1STAT = 0b10000000;     // スルーレート制御はOff
    SSP1ADD = (_XTAL_FREQ/(4*baud))-1;      //  SSP1ADD = 0x77;
    SSP1CON1 = 0b00101000;     // I2C Master modeにする
}



/********************************************************
 * i2c Library Master Mode Only
*********************************************************/
char i2c_Write(char data){
    SSPBUF = data;                 // データセット
    PIR1bits.SSP1IF = 0;           // 終了フラグクリア
    while(!PIR1bits.SSP1IF);       // 送信終了待ち
    return SSPCON2bits.ACKSTAT;    // ACKなら「0」で復帰
}
char i2c_Read( void )
{
  SSPCON2bits.RCEN = 1;            // データ受信を指示
  while ( !SSPSTATbits.BF );       // 8ビット受信の完了を待つ
  return  SSPBUF;                  // 受信データで復帰
}
/**************************************************************
*    EEPROM 内部アドレスポインタをセットする
*    指定アドレスのメモリーブロックを戻値で返す
*    メモリーブロック　0:00000-0FFFF  1:10000-1FFFF
 **************************************************************/
char EEPROM_AddressSet(unsigned long add){
    char block =  (add > 0xFFFF);
    i2c_Idle();                       // busアイドルの確認
    i2c_Start();                      // Start bit
    i2c_Write(SlaveAdd_W | block*8);  // 制御コード送信
    i2c_Write(add >> 8);              // アドレス送信
    i2c_Write((char)add);             // アドレス送信
    i2c_Stop();                       // Stop bit
    return block;                     //
}

/**************************************************************
 *    EEPROM 内部アドレスポインタの示すデータを読み出す
 * 　　　読み出し後にアドレスポインタを +1 する
 * 　　　引数：メモリブロック番号
 **************************************************************/
char EEPROM_ReadCurrent(char block){
    char Data;
    i2c_Idle();                       // busアイドルの確認
    i2c_Start();                      // Start bit
    i2c_Write(SlaveAdd_R | block*8);  // 制御コード送信
    Data = i2c_Read();                // データ受信
    i2c_NACK();                       // NACK送信
    i2c_Stop();                       // Stop bit
     return Data;
}
